@@ -8,6 +8,8 @@ import sys
 import threading
 import time
 
+from .util import six
+
 # Import some platform-specific things at top level so they can be mocked for
 # tests.
 try:
@@ -26,15 +28,10 @@ except ImportError:
 from .exceptions import (
     UnexpectedExit, Failure, ThreadException, WatcherError,
 )
-from .platform import (
+from .terminals import (
     WINDOWS, pty_size, character_buffered, ready_for_reading, bytes_to_read,
 )
-from .util import has_fileno, isatty, ExceptionHandlingThread
-
-try:
-    from .vendor import six
-except ImportError:
-    import six
+from .util import has_fileno, isatty, ExceptionHandlingThread, encode_output
 
 
 class Runner(object):
@@ -44,6 +41,8 @@ class Runner(object):
     This class is not usable by itself and must be subclassed, implementing a
     number of methods such as `start`, `wait` and `returncode`. For a subclass
     implementation example, see the source code for `.Local`.
+
+    .. versionadded:: 1.0
     """
     read_chunk_size = 1000
     input_sleep = 0.01
@@ -103,7 +102,9 @@ class Runner(object):
 
         :param str command: The shell command to execute.
 
-        :param str shell: Which shell binary to use. Default: ``/bin/bash``.
+        :param str shell:
+            Which shell binary to use. Default: ``/bin/bash`` (on Unix;
+            ``COMSPEC`` or ``cmd.exe`` on Windows.)
 
         :param bool warn:
             Whether to warn and continue, instead of raising
@@ -189,7 +190,7 @@ class Runner(object):
             of `default_encoding`).
 
         :param out_stream:
-            A file-like stream object to which the subprocess' standard error
+            A file-like stream object to which the subprocess' standard output
             should be written. If ``None`` (the default), ``sys.stdout`` will
             be used.
 
@@ -201,7 +202,13 @@ class Runner(object):
             A file-like stream object to used as the subprocess' standard
             input. If ``None`` (the default), ``sys.stdin`` will be used.
 
-        :param list watchers:
+            If ``False``, will disable stdin mirroring entirely (though other
+            functionality which writes to the subprocess' stdin, such as
+            autoresponding, will still function.) Disabling stdin mirroring can
+            help when ``sys.stdin`` is a misbehaving non-stream object, such as
+            under test harnesses or headless command runners.
+
+        :param watchers:
             A list of `.StreamWatcher` instances which will be used to scan the
             program's ``stdout`` or ``stderr`` and may write into its ``stdin``
             (typically ``str`` or ``bytes`` objects depending on Python
@@ -254,6 +261,8 @@ class Runner(object):
         :raises:
             `.ThreadException` (if the background I/O threads encountered
             exceptions other than `.WatcherError`).
+
+        .. versionadded:: 1.0
         """
         try:
             return self._run_body(command, **kwargs)
@@ -268,7 +277,7 @@ class Runner(object):
         env = self.generate_env(opts['env'], opts['replace_env'])
         # Echo running command
         if opts['echo']:
-            print("\033[1;37m{0}\033[0m".format(command))
+            print("\033[1;37m{}\033[0m".format(command))
         # Start executing the actual command (runs in background)
         self.start(command, shell, env)
         # Arrive at final encoding if neither config nor kwargs had one
@@ -281,17 +290,17 @@ class Runner(object):
                 'hide': 'stdout' in opts['hide'],
                 'output': out_stream,
             },
-            # TODO: make this & related functionality optional, for users who
-            # don't care about autoresponding & are encountering issues with
-            # the stdin mirroring? Downside is it fragments expected behavior &
-            # puts folks with true interactive use cases in a different support
-            # class.
-            self.handle_stdin: {
+        }
+        # After opt processing above, in_stream will be a real stream obj or
+        # False, so we can truth-test it. We don't even create a stdin-handling
+        # thread if it's False, meaning user indicated stdin is nonexistent or
+        # problematic.
+        if in_stream:
+            thread_args[self.handle_stdin] = {
                 'input_': in_stream,
                 'output': out_stream,
                 'echo': opts['echo_stdin'],
             }
-        }
         if not self.using_pty:
             thread_args[self.handle_stderr] = {
                 'buffer_': stderr,
@@ -299,10 +308,11 @@ class Runner(object):
                 'output': err_stream,
             }
         # Kick off IO threads
-        self.threads, exceptions = [], []
+        self.threads = {}
+        exceptions = []
         for target, kwargs in six.iteritems(thread_args):
             t = ExceptionHandlingThread(target=target, kwargs=kwargs)
-            self.threads.append(t)
+            self.threads[target] = t
             t.start()
         # Wait for completion, then tie things off & obtain result
         # And make sure we perform that tying off even if things asplode.
@@ -324,21 +334,12 @@ class Runner(object):
                 exception = e
                 # Break out of return-to-wait() loop - we want to shut down
                 break
+        # Inform stdin-mirroring worker to stop its eternal looping
         self.program_finished.set()
-        for t in self.threads:
-            # NOTE: using a join timeout for corner case from #351 (one pipe
-            # excepts, fills up, prevents subproc from exiting, and other pipe
-            # then has a blocking read() call, causing its thread to block on
-            # join). In normal, non-#351 situations this should function
-            # similarly to a non-timeout'd join.
-            # NOTE: but we avoid a timeout for the stdin handler as it has its
-            # own termination conditions & isn't subject to this corner case.
-            timeout = None
-            if t.kwargs['target'] != self.handle_stdin:
-                # TODO: make the timeout configurable
-                timeout = 1
-            t.join(timeout)
-            e = t.exception()
+        # Join threads, setting a timeout if necessary
+        for target, thread in six.iteritems(self.threads):
+            thread.join(self._thread_timeout(target))
+            e = thread.exception()
             if e is not None:
                 exceptions.append(e)
         # If we got a main-thread exception while wait()ing, raise it now that
@@ -387,6 +388,7 @@ class Runner(object):
             exited=exited,
             pty=self.using_pty,
             hide=opts['hide'],
+            encoding=self.encoding,
         )
         # Any presence of WatcherError from the threads indicates a watcher was
         # upset and aborted execution; make a generic Failure out of it and
@@ -414,7 +416,7 @@ class Runner(object):
         # Handle invalid kwarg keys (anything left in kwargs).
         # Act like a normal function would, i.e. TypeError
         if kwargs:
-            err = "run() got an unexpected keyword argument '{0}'"
+            err = "run() got an unexpected keyword argument '{}'"
             raise TypeError(err.format(list(kwargs.keys())[0]))
         # If hide was True, turn off echoing
         if opts['hide'] is True:
@@ -438,6 +440,22 @@ class Runner(object):
             self.watchers = opts['watchers']
         return opts, out_stream, err_stream, in_stream
 
+    def _thread_timeout(self, target):
+        # Add a timeout to out/err thread joins when it looks like they're not
+        # dead but their counterpart is dead; this indicates issue #351 (fixed
+        # by #432) where the subproc may hang because its stdout (or stderr) is
+        # no longer being consumed by the dead thread (and a pipe is filling
+        # up.) In that case, the non-dead thread is likely to block forever on
+        # a `recv` unless we add this timeout.
+        if target == self.handle_stdin:
+            return None
+        opposite = self.handle_stderr
+        if target == self.handle_stderr:
+            opposite = self.handle_stdout
+        if opposite in self.threads and self.threads[opposite].is_dead:
+            return 1
+        return None
+
     def generate_result(self, **kwargs):
         """
         Create & return a suitable `Result` instance from the given ``kwargs``.
@@ -445,6 +463,8 @@ class Runner(object):
         Subclasses may wish to override this in order to manipulate things or
         generate a `Result` subclass (e.g. ones containing additional metadata
         besides the default).
+
+        .. versionadded:: 1.0
         """
         return Result(**kwargs)
 
@@ -467,6 +487,8 @@ class Runner(object):
 
             Specifically, each resulting string is the result of decoding
             `read_chunk_size` bytes read from the subprocess' out/err stream.
+
+        .. versionadded:: 1.0
         """
         # NOTE: Typically, reading from any stdout/err (local, remote or
         # otherwise) can be thought of as "read until you get nothing back".
@@ -494,20 +516,10 @@ class Runner(object):
         :param string: A Unicode string object.
 
         :returns: ``None``.
+
+        .. versionadded:: 1.0
         """
-        # Encode under Python 2 only, because of the common problem where
-        # sys.stdout/err on Python 2 end up using sys.getdefaultencoding(),
-        # which is frequently NOT the same thing as the real local terminal
-        # encoding (reflected as sys.stdout.encoding). I.e. even when
-        # sys.stdout.encoding is UTF-8, ascii is still actually used, and
-        # explodes.
-        # Python 3 doesn't have this problem, so we delegate encoding to the
-        # io.*Writer classes involved.
-        if six.PY2:
-            # TODO: split up self.encoding, only use the one for 'local
-            # encoding' here.
-            string = string.encode(self.encoding)
-        stream.write(string)
+        stream.write(encode_output(string, self.encoding))
         stream.flush()
 
     def _handle_output(self, buffer_, hide, output, reader):
@@ -535,13 +547,15 @@ class Runner(object):
         Intended for use as a thread target. Only terminates when all stdout
         from the subprocess has been read.
 
-        :param list buffer_: The capture buffer shared with the main thread.
+        :param buffer_: The capture buffer shared with the main thread.
         :param bool hide: Whether or not to replay data into ``output``.
         :param output:
             Output stream (file-like object) to write data into when not
             hiding.
 
         :returns: ``None``.
+
+        .. versionadded:: 1.0
         """
         self._handle_output(
             buffer_,
@@ -556,6 +570,8 @@ class Runner(object):
 
         Identical to `handle_stdout` except for the stream read from; see its
         docstring for API details.
+
+        .. versionadded:: 1.0
         """
         self._handle_output(
             buffer_,
@@ -577,6 +593,8 @@ class Runner(object):
             A Unicode string, the result of decoding the read bytes (this might
             be the empty string if the pipe has closed/reached EOF); or
             ``None`` if stdin wasn't ready for reading yet.
+
+        .. versionadded:: 1.0
         """
         # TODO: consider moving the character_buffered contextmanager call in
         # here? Downside is it would be flipping those switches for every byte
@@ -613,6 +631,8 @@ class Runner(object):
         :param bool echo: User override option for stdin-stdout echoing.
 
         :returns: ``None``.
+
+        .. versionadded:: 1.0
         """
         # TODO: reinstate lock/whatever thread logic from fab v1 which prevents
         # reading from stdin while other parts of the code are prompting for
@@ -658,6 +678,8 @@ class Runner(object):
         :param input_: Input stream (file-like object).
         :param output: Output stream (file-like object).
         :returns: A ``bool``.
+
+        .. versionadded:: 1.0
         """
         return (not self.using_pty) and isatty(input_)
 
@@ -669,10 +691,12 @@ class Runner(object):
         from the ``watchers`` kwarg of `run` - see :doc:`/concepts/watchers`
         for a conceptual overview.
 
-        :param list buffer:
+        :param buffer:
             The capture buffer for this thread's particular IO stream.
 
         :returns: ``None``.
+
+        .. versionadded:: 1.0
         """
         # Join buffer contents into a single string; without this,
         # StreamWatcher subclasses can't do things like iteratively scan for
@@ -696,6 +720,8 @@ class Runner(object):
             `os.environ`.
 
         :returns: A dictionary of shell environment vars.
+
+        .. versionadded:: 1.0
         """
         return env if replace_env else dict(os.environ, **env)
 
@@ -708,6 +734,8 @@ class Runner(object):
         :param bool fallback:
             Whether falling back to non-pty execution should be allowed, in
             situations where ``pty=True`` but a pty could not be allocated.
+
+        .. versionadded:: 1.0
         """
         # NOTE: fallback not used: no falling back implemented by default.
         return pty
@@ -724,14 +752,18 @@ class Runner(object):
         :returns:
             ``True`` if any threads appear to have terminated with an
             exception, ``False`` otherwise.
+
+        .. versionadded:: 1.0
         """
-        return any(x.is_dead for x in self.threads)
+        return any(x.is_dead for x in self.threads.values())
 
     def wait(self):
         """
         Block until the running command appears to have exited.
 
         :returns: ``None``.
+
+        .. versionadded:: 1.0
         """
         while True:
             proc_finished = self.process_is_finished
@@ -747,6 +779,8 @@ class Runner(object):
         :param data: A Unicode string.
 
         :returns: ``None``.
+
+        .. versionadded:: 1.0
         """
         # Encode always, then request implementing subclass to perform the
         # actual write to subprocess' stdin.
@@ -755,6 +789,8 @@ class Runner(object):
     def decode(self, data):
         """
         Decode some ``data`` bytes, returning Unicode.
+
+        .. versionadded:: 1.0
         """
         # NOTE: yes, this is a 1-liner. The point is to make it much harder to
         # forget to use 'replace' when decoding :)
@@ -772,6 +808,8 @@ class Runner(object):
         :returns:
             ``True`` if the subprocess has finished running, ``False``
             otherwise.
+
+        .. versionadded:: 1.0
         """
         raise NotImplementedError
 
@@ -784,6 +822,8 @@ class Runner(object):
 
         In most cases, this method will also set subclass-specific member
         variables used in other methods such as `wait` and/or `returncode`.
+
+        .. versionadded:: 1.0
         """
         raise NotImplementedError
 
@@ -794,6 +834,8 @@ class Runner(object):
         :param int num_bytes: Number of bytes to read at maximum.
 
         :returns: A string/bytes object.
+
+        .. versionadded:: 1.0
         """
         raise NotImplementedError
 
@@ -804,6 +846,8 @@ class Runner(object):
         :param int num_bytes: Number of bytes to read at maximum.
 
         :returns: A string/bytes object.
+
+        .. versionadded:: 1.0
         """
         raise NotImplementedError
 
@@ -817,6 +861,8 @@ class Runner(object):
         :param data: Already-encoded byte data suitable for writing.
 
         :returns: ``None``.
+
+        .. versionadded:: 1.0
         """
         raise NotImplementedError
 
@@ -825,6 +871,8 @@ class Runner(object):
         Return a string naming the expected encoding of subprocess streams.
 
         This return value should be suitable for use by encode/decode methods.
+
+        .. versionadded:: 1.0
         """
         # TODO: probably wants to be 2 methods, one for local and one for
         # subprocess. For now, good enough to assume both are the same.
@@ -853,6 +901,8 @@ class Runner(object):
             The locally-sourced ``KeyboardInterrupt`` causing the method call.
 
         :returns: ``None``.
+
+        .. versionadded:: 1.0
         """
         self.write_proc_stdin(u'\x03')
 
@@ -861,6 +911,8 @@ class Runner(object):
         Return the numeric return/exit code resulting from command execution.
 
         :returns: `int`
+
+        .. versionadded:: 1.0
         """
         raise NotImplementedError
 
@@ -873,6 +925,8 @@ class Runner(object):
         things such as close network connections or open files.
 
         :returns: ``None``
+
+        .. versionadded:: 1.0
         """
         raise NotImplementedError
 
@@ -890,6 +944,8 @@ class Local(Runner):
         a warning to stderr.
 
         To disable this behavior, say ``fallback=False``.
+
+    .. versionadded:: 1.0
     """
     def __init__(self, context):
         super(Local, self).__init__(context)
@@ -915,8 +971,15 @@ class Local(Runner):
             try:
                 data = os.read(self.parent_fd, num_bytes)
             except OSError as e:
-                # Only eat this specific OSError so we don't hide others
-                if "Input/output error" not in str(e):
+                # Only eat I/O specific OSErrors so we don't hide others
+                stringified = str(e)
+                io_errors = (
+                    # The typical default
+                    "Input/output error",
+                    # Some less common platforms phrase it this way
+                    "I/O error",
+                )
+                if not any(error in stringified for error in io_errors):
                     raise
                 # The bad OSErrors happen after all expected output has
                 # appeared, so we return a falsey value, which triggers the
@@ -1036,6 +1099,9 @@ class Result(object):
         was invoked via a pty, in which case it will be empty; see
         `.Runner.run`.)
 
+    :param str encoding:
+        The string encoding used by the local shell environment.
+
     :param str command:
         The command which was executed.
 
@@ -1076,12 +1142,15 @@ class Result(object):
 
         However, remember `Zen of Python #2
         <http://zen-of-python.info/explicit-is-better-than-implicit.html#2>`_.
+
+    .. versionadded:: 1.0
     """
     # TODO: inherit from namedtuple instead? heh (or: use attrs from pypi)
     def __init__(
         self,
         stdout="",
         stderr="",
+        encoding=None,
         command="",
         shell="",
         env=None,
@@ -1091,6 +1160,7 @@ class Result(object):
     ):
         self.stdout = stdout
         self.stderr = stderr
+        self.encoding = encoding
         self.command = command
         self.shell = shell
         self.env = {} if env is None else env
@@ -1102,6 +1172,8 @@ class Result(object):
     def return_code(self):
         """
         An alias for ``.exited``.
+
+        .. versionadded:: 1.0
         """
         return self.exited
 
@@ -1117,21 +1189,30 @@ class Result(object):
 
     def __str__(self):
         if self.exited is not None:
-            desc = "Command exited with status {0}.".format(self.exited)
+            desc = "Command exited with status {}.".format(self.exited)
         else:
             desc = "Command was not fully executed due to watcher error."
         ret = [desc]
         for x in ('stdout', 'stderr'):
             val = getattr(self, x)
-            ret.append(u"""=== {0} ===
-{1}
-""".format(x, val.rstrip()) if val else u"(no {0})".format(x))
+            ret.append(u"""=== {} ===
+{}
+""".format(x, val.rstrip()) if val else u"(no {})".format(x))
         return u"\n".join(ret)
 
+    def __repr__(self):
+        # TODO: more? e.g. len of stdout/err? (how to represent cleanly in a
+        # 'x=y' format like this? e.g. '4b' is ambiguous as to what it
+        # represents
+        template = "<Result cmd={!r} exited={}>"
+        return template.format(self.command, self.exited)
+
     @property
     def ok(self):
         """
         A boolean equivalent to ``exited == 0``.
+
+        .. versionadded:: 1.0
         """
         return self.exited == 0
 
@@ -1142,6 +1223,8 @@ class Result(object):
 
         I.e., ``True`` if the program exited with a nonzero return code, and
         ``False`` otherwise.
+
+        .. versionadded:: 1.0
         """
         return not self.ok
 
@@ -1149,7 +1232,7 @@ class Result(object):
 def normalize_hide(val):
     hide_vals = (None, False, 'out', 'stdout', 'err', 'stderr', 'both', True)
     if val not in hide_vals:
-        err = "'hide' got {0!r} which is not in {1!r}"
+        err = "'hide' got {!r} which is not in {!r}"
         raise ValueError(err.format(val, hide_vals))
     if val in (None, False):
         hide = ()
