@@ -1,11 +1,12 @@
 from datetime import datetime
+from os.path import abspath, join, dirname
 
 import alabaster
 
 
 # Alabaster theme + mini-extension
 html_theme_path = [alabaster.get_path()]
-extensions = ['alabaster', 'sphinx.ext.intersphinx']
+extensions = ['alabaster', 'sphinx.ext.intersphinx', 'sphinx.ext.doctest']
 # Paths relative to invoking conf.py - not this shared file
 html_theme = 'alabaster'
 html_theme_options = {
@@ -27,13 +28,19 @@ html_sidebars = {
 
 # Everything intersphinx's to Python
 intersphinx_mapping = {
-    'python': ('http://docs.python.org/2.6', None),
+    'python': ('https://docs.python.org/2.7/', None),
 }
 
+# Doctest settings
+doctest_path = [abspath(join(dirname(__file__), '..', 'tests'))]
+doctest_global_setup = r"""
+from _util import MockSubprocess
+"""
+
 # Regular settings
 project = 'Invoke'
 year = datetime.now().year
-copyright = '%d Jeff Forcier' % year
+copyright = '{} Jeff Forcier'.format(year)
 master_doc = 'index'
 templates_path = ['_templates']
 exclude_trees = ['_build']
