@@ -1,11 +1,9 @@
-from spec import eq_
-
 from invoke import task, Collection
 
 
 @task
 def mytask(c):
-    eq_(c.outer.inner.hooray, 'yaml')
+    assert c.outer.inner.hooray == 'yaml'
 
 
 ns = Collection(mytask)
