@@ -3,11 +3,11 @@ from invoke import Collection, task, call
 from package import module
 
 @task
-def top_pre(ctx):
+def top_pre(c):
     pass
 
 @task(call(top_pre))
-def toplevel(ctx):
+def toplevel(c):
     pass
 
 ns = Collection(module, toplevel)
