@@ -2,11 +2,11 @@ from invoke import task
 
 
 @task
-def no_docstring(ctx):
+def no_docstring(c):
     pass
 
 @task
-def task_one(ctx):
+def task_one(c):
     """
     Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam id dictum
 
@@ -14,7 +14,7 @@ def task_one(ctx):
     """
 
 @task
-def task_two(ctx):
+def task_two(c):
     """
     Nulla eget ultrices ante. Curabitur sagittis commodo posuere. Duis dapibus
 
