@@ -1,11 +1,11 @@
 from invoke import task, Collection
 
 @task
-def z_toplevel(ctx):
+def z_toplevel(c):
     pass
 
 @task
-def subtask(ctx):
+def subtask(c):
     pass
 
 ns = Collection(z_toplevel, Collection('a', Collection('b', subtask)))
