@@ -1,9 +1,9 @@
-from spec import Spec, eq_, skip, ok_, raises
+from pytest import skip, raises
 
 from invoke.parser import Argument
 
 
-class Argument_(Spec):
+class Argument_:
     class init:
         "__init__"
         def may_take_names_list(self):
@@ -16,63 +16,50 @@ class Argument_(Spec):
         def may_take_name_arg(self):
             assert '-b' in Argument(name='-b').names
 
-        @raises(TypeError)
         def must_get_at_least_one_name(self):
-            Argument()
+            with raises(TypeError):
+                Argument()
 
         def default_arg_is_name_not_names(self):
             assert 'b' in Argument('b').names
 
         def can_declare_positional(self):
-            eq_(Argument(name='foo', positional=True).positional, True)
+            assert Argument(name='foo', positional=True).positional is True
 
         def positional_is_False_by_default(self):
-            eq_(Argument(name='foo').positional, False)
+            assert Argument(name='foo').positional is False
 
         def can_set_attr_name_to_control_name_attr(self):
             a = Argument('foo', attr_name='bar')
-            eq_(a.name, 'bar') # not 'foo'
+            assert a.name == 'bar' # not 'foo'
 
-    class string:
-        "__str__"
+    class repr:
+        "__repr__"
 
         def shows_useful_info(self):
-            eq_(
-                str(Argument(names=('name', 'nick1', 'nick2'))),
-                "<Argument: {0} ({1})>".format('name', 'nick1, nick2')
-            )
+            arg = Argument(names=('name', 'nick1', 'nick2'))
+            expected = "<Argument: {} ({})>".format('name', 'nick1, nick2')
+            assert repr(arg) == expected
 
         def does_not_show_nickname_parens_if_no_nicknames(self):
-            eq_(
-                str(Argument('name')),
-                "<Argument: name>"
-            )
+            assert repr(Argument('name')) == "<Argument: name>"
 
         def shows_positionalness(self):
-            eq_(
-                str(Argument('name', positional=True)),
-                "<Argument: name *>"
-            )
+            arg = Argument('name', positional=True)
+            assert repr(arg) == "<Argument: name *>"
 
         def shows_optionalness(self):
-            eq_(
-                str(Argument('name', optional=True)),
-                "<Argument: name ?>",
-            )
+            arg = Argument('name', optional=True)
+            assert repr(arg) == "<Argument: name ?>"
 
         def positionalness_and_optionalness_stick_together(self):
             # TODO: but do these even make sense on the same argument? For now,
             # best to have a nonsensical test than a missing one...
-            eq_(
-                str(Argument('name', optional=True, positional=True)),
-                "<Argument: name *?>",
-            )
+            arg = Argument('name', optional=True, positional=True)
+            assert repr(arg) == "<Argument: name *?>"
 
         def shows_kind_if_not_str(self):
-            eq_(
-                str(Argument('age', kind=int)),
-                "<Argument: age [int]>",
-            )
+            assert repr(Argument('age', kind=int)) == "<Argument: age [int]>"
 
         def all_the_things_together(self):
             arg = Argument(
@@ -81,14 +68,7 @@ class Argument_(Spec):
                 optional=True,
                 positional=True,
             )
-            eq_(str(arg), "<Argument: meh (m) [int] *?>")
-
-    class repr:
-        "__repr__"
-
-        def just_aliases_dunder_str(self):
-            a = Argument(names=('name', 'name2'))
-            eq_(str(a), repr(a))
+            assert repr(arg) == "<Argument: meh (m) [int] *?>"
 
     class kind_kwarg:
         "'kind' kwarg"
@@ -98,10 +78,12 @@ class Argument_(Spec):
             Argument(name='b', kind=int)
 
         def defaults_to_str(self):
-            eq_(Argument('a').kind, str)
+            assert Argument('a').kind == str
 
         def non_bool_implies_value_needed(self):
             assert Argument(name='a', kind=int).takes_value
+            assert Argument(name='b', kind=str).takes_value
+            assert Argument(name='c', kind=list).takes_value
 
         def bool_implies_no_value_needed(self):
             assert not Argument(name='a', kind=bool).takes_value
@@ -114,25 +96,28 @@ class Argument_(Spec):
             # ./configure)
             skip()
 
-        @raises(ValueError)
         def may_validate_on_set(self):
-            Argument('a', kind=int).value = 'five'
+            with raises(ValueError):
+                Argument('a', kind=int).value = 'five'
+
+        def list_implies_initial_value_of_empty_list(self):
+            assert Argument('mylist', kind=list).value == []
 
     class names:
         def returns_tuple_of_all_names(self):
-            eq_(Argument(names=('--foo', '-b')).names, ('--foo', '-b'))
-            eq_(Argument(name='--foo').names, ('--foo',))
+            assert Argument(names=('--foo', '-b')).names == ('--foo', '-b')
+            assert Argument(name='--foo').names == ('--foo',)
 
         def is_normalized_to_a_tuple(self):
-            ok_(isinstance(Argument(names=('a', 'b')).names, tuple))
+            assert isinstance(Argument(names=('a', 'b')).names, tuple)
 
     class name:
         def returns_first_name(self):
-            eq_(Argument(names=('a', 'b')).name, 'a')
+            assert Argument(names=('a', 'b')).name == 'a'
 
     class nicknames:
         def returns_rest_of_names(self):
-            eq_(Argument(names=('a', 'b')).nicknames, ('b',))
+            assert Argument(names=('a', 'b')).nicknames == ('b',)
 
     class takes_value:
         def True_by_default(self):
@@ -147,38 +132,61 @@ class Argument_(Spec):
             "available as .raw_value"
             a = Argument('a')
             a.value = 'foo'
-            eq_(a.raw_value, 'foo')
+            assert a.raw_value == 'foo'
 
         def untransformed_appears_as_dot_value(self):
             "untransformed, appears as .value"
             a = Argument('a', kind=str)
             a.value = 'foo'
-            eq_(a.value, 'foo')
+            assert a.value == 'foo'
 
         def transformed_appears_as_dot_value_with_original_as_raw_value(self):
             "transformed, modified value is .value, original is .raw_value"
             a = Argument('a', kind=int)
             a.value = '5'
-            eq_(a.value, 5)
-            eq_(a.raw_value, '5')
+            assert a.value == 5
+            assert a.raw_value == '5'
+
+        def list_kind_triggers_append_instead_of_overwrite(self):
+            # TODO: when put this way it makes the API look pretty strange;
+            # maybe a sign we should switch to explicit setter methods
+            # (selected on kind, perhaps) instead of using an implicit setter
+            a = Argument('mylist', kind=list)
+            assert a.value == []
+            a.value = 'val1'
+            assert a.value == ['val1']
+            a.value = 'val2'
+            assert a.value == ['val1', 'val2']
+
+        def incrementable_True_triggers_increment_of_default(self):
+            a = Argument('verbose', kind=int, default=0, incrementable=True)
+            assert a.value == 0
+            # NOTE: parser currently just goes "Argument.takes_value is false?
+            # Gonna stuff True/False in there." So this looks pretty silly out
+            # of context (as with list-types above.)
+            a.value = True
+            assert a.value == 1
+            for _ in range(4):
+                a.value = True
+            assert a.value == 5
 
     class value:
         def returns_default_if_not_set(self):
             a = Argument('a', default=25)
-            eq_(a.value, 25)
+            assert a.value == 25
 
     class raw_value:
         def is_None_when_no_value_was_actually_seen(self):
             a = Argument('a', kind=int)
-            eq_(a.raw_value, None)
+            assert a.raw_value is None
 
     class set_value:
         def casts_by_default(self):
             a = Argument('a', kind=int)
             a.set_value('5')
-            eq_(a.value, 5)
+            assert a.value == 5
 
         def allows_setting_value_without_casting(self):
             a = Argument('a', kind=int)
             a.set_value('5', cast=False)
-            eq_(a.value, '5')
+            assert a.value == '5'
