@@ -1,53 +1,53 @@
-from spec import Spec, eq_, raises
+from pytest import raises, skip
 
 from invoke.parser import Parser, Context, Argument, ParseError
 
 
-class Parser_(Spec):
+class Parser_:
     def can_take_initial_context(self):
         c = Context()
         p = Parser(initial=c)
-        eq_(p.initial, c)
+        assert p.initial == c
 
     def can_take_initial_and_other_contexts(self):
         c1 = Context('foo')
         c2 = Context('bar')
         p = Parser(initial=Context(), contexts=[c1, c2])
-        eq_(p.contexts['foo'], c1)
-        eq_(p.contexts['bar'], c2)
+        assert p.contexts['foo'] == c1
+        assert p.contexts['bar'] == c2
 
     def can_take_just_other_contexts(self):
         c = Context('foo')
         p = Parser(contexts=[c])
-        eq_(p.contexts['foo'], c)
+        assert p.contexts['foo'] == c
 
     def can_take_just_contexts_as_non_keyword_arg(self):
         c = Context('foo')
         p = Parser([c])
-        eq_(p.contexts['foo'], c)
+        assert p.contexts['foo'] == c
 
-    @raises(ValueError)
     def raises_ValueError_for_unnamed_Contexts_in_contexts(self):
-        Parser(initial=Context(), contexts=[Context()])
+        with raises(ValueError):
+            Parser(initial=Context(), contexts=[Context()])
 
-    @raises(ValueError)
     def raises_error_for_context_name_clashes(self):
-        Parser(contexts=(Context('foo'), Context('foo')))
+        with raises(ValueError):
+            Parser(contexts=(Context('foo'), Context('foo')))
 
-    @raises(ValueError)
     def raises_error_for_context_alias_and_name_clashes(self):
-        Parser((Context('foo', aliases=('bar',)), Context('bar')))
+        with raises(ValueError):
+            Parser((Context('foo', aliases=('bar',)), Context('bar')))
 
-    @raises(ValueError)
     def raises_error_for_context_name_and_alias_clashes(self):
         # I.e. inverse of the above, which is a different code path.
-        Parser((Context('foo'), Context('bar', aliases=('foo',))))
+        with raises(ValueError):
+            Parser((Context('foo'), Context('bar', aliases=('foo',))))
 
     def takes_ignore_unknown_kwarg(self):
         Parser(ignore_unknown=True)
 
     def ignore_unknown_defaults_to_False(self):
-        eq_(Parser().ignore_unknown, False)
+        assert Parser().ignore_unknown is False
 
     class parse_argv:
         def parses_sys_argv_style_list_of_strings(self):
@@ -62,61 +62,61 @@ class Parser_(Spec):
             task1 = Context('mytask')
             task2 = Context('othertask')
             result = Parser((task1, task2)).parse_argv(['othertask'])
-            eq_(len(result), 1)
-            eq_(result[0].name, 'othertask')
+            assert len(result) == 1
+            assert result[0].name == 'othertask'
 
-        @raises(ParseError)
         def raises_error_if_unknown_contexts_found(self):
-            Parser().parse_argv(['foo', 'bar'])
+            with raises(ParseError):
+                Parser().parse_argv(['foo', 'bar'])
 
         def unparsed_does_not_share_state(self):
             r = Parser(ignore_unknown=True).parse_argv(['self'])
-            eq_(r.unparsed, ['self'])
+            assert r.unparsed == ['self']
             r2 = Parser(ignore_unknown=True).parse_argv(['contained'])
-            eq_(r.unparsed, ['self']) # NOT ['self', 'contained']
-            eq_(r2.unparsed, ['contained']) # NOT ['self', 'contained']
+            assert r.unparsed == ['self'] # NOT ['self', 'contained']
+            assert r2.unparsed == ['contained'] # NOT ['self', 'contained']
 
         def ignore_unknown_returns_unparsed_argv_instead(self):
             r = Parser(ignore_unknown=True).parse_argv(['foo', 'bar', '--baz'])
-            eq_(r.unparsed, ['foo', 'bar', '--baz'])
+            assert r.unparsed == ['foo', 'bar', '--baz']
 
         def ignore_unknown_does_not_mutate_rest_of_argv(self):
             p = Parser([Context('ugh')], ignore_unknown=True)
             r = p.parse_argv(['ugh', 'what', '-nowai'])
             # NOT: ['what', '-n', '-w', '-a', '-i']
-            eq_(r.unparsed, ['what', '-nowai'])
+            assert r.unparsed == ['what', '-nowai']
 
         def always_includes_initial_context_if_one_was_given(self):
             # Even if no core/initial flags were seen
             t1 = Context('t1')
             init = Context()
             result = Parser((t1,), initial=init).parse_argv(['t1'])
-            eq_(result[0].name, None)
-            eq_(result[1].name, 't1')
+            assert result[0].name is None
+            assert result[1].name == 't1'
 
         def returned_contexts_are_in_order_given(self):
             t1, t2 = Context('t1'), Context('t2')
             r = Parser((t1, t2)).parse_argv(['t2', 't1'])
-            eq_([x.name for x in r], ['t2', 't1'])
+            assert [x.name for x in r] == ['t2', 't1']
 
         def returned_context_member_arguments_contain_given_values(self):
             c = Context('mytask', args=(Argument('boolean', kind=bool),))
             result = Parser((c,)).parse_argv(['mytask', '--boolean'])
-            eq_(result[0].args['boolean'].value, True)
+            assert result[0].args['boolean'].value is True
 
         def inverse_bools_get_set_correctly(self):
             arg = Argument('myarg', kind=bool, default=True)
             c = Context('mytask', args=(arg,))
             r = Parser((c,)).parse_argv(['mytask', '--no-myarg'])
-            eq_(r[0].args['myarg'].value, False)
+            assert r[0].args['myarg'].value is False
 
         def arguments_which_take_values_get_defaults_overridden_correctly(self): # noqa
             args = (Argument('arg', kind=str), Argument('arg2', kind=int))
             c = Context('mytask', args=args)
             argv = ['mytask', '--arg', 'myval', '--arg2', '25']
             result = Parser((c,)).parse_argv(argv)
-            eq_(result[0].args['arg'].value, 'myval')
-            eq_(result[0].args['arg2'].value, 25)
+            assert result[0].args['arg'].value == 'myval'
+            assert result[0].args['arg2'].value == 25
 
         def returned_arguments_not_given_contain_default_values(self):
             # I.e. a Context with args A and B, invoked with no mention of B,
@@ -126,18 +126,18 @@ class Parser_(Spec):
             b = Argument('age', default=7)
             c = Context('mytask', args=(a, b))
             Parser((c,)).parse_argv(['mytask', '--name', 'blah'])
-            eq_(c.args['age'].value, 7)
+            assert c.args['age'].value == 7
 
         def returns_remainder(self):
             "returns -- style remainder string chunk"
             r = Parser((Context('foo'),)).parse_argv(
                 ['foo', '--', 'bar', 'biz']
             )
-            eq_(r.remainder, "bar biz")
+            assert r.remainder == "bar biz"
 
         def clones_initial_context(self):
             a = Argument('foo', kind=bool)
-            eq_(a.value, None)
+            assert a.value is None
             c = Context(args=(a,))
             p = Parser(initial=c)
             assert p.initial is c
@@ -147,12 +147,12 @@ class Parser_(Spec):
             assert c2 is not c
             a2 = c2.args['foo']
             assert a2 is not a
-            eq_(a.value, None)
-            eq_(a2.value, True)
+            assert a.value is None
+            assert a2.value is True
 
         def clones_noninitial_contexts(self):
             a = Argument('foo')
-            eq_(a.value, None)
+            assert a.value is None
             c = Context(name='mytask', args=(a,))
             p = Parser(contexts=(c,))
             assert p.contexts['mytask'] is c
@@ -162,16 +162,16 @@ class Parser_(Spec):
             assert c2 is not c
             a2 = c2.args['foo']
             assert a2 is not a
-            eq_(a.value, None)
-            eq_(a2.value, 'val')
+            assert a.value is None
+            assert a2.value == 'val'
 
         class parsing_errors:
             def setup(self):
                 self.p = Parser([Context(name='foo', args=[Argument('bar')])])
 
-            @raises(ParseError)
             def missing_flag_values_raise_ParseError(self):
-                self.p.parse_argv(['foo', '--bar'])
+                with raises(ParseError):
+                    self.p.parse_argv(['foo', '--bar'])
 
             def attaches_context_to_ParseErrors(self):
                 try:
@@ -193,11 +193,26 @@ class Parser_(Spec):
 
             def single_positional_arg(self):
                 r = self._basic().parse_argv(['mytask', 'posval'])
-                eq_(r[0].args['pos'].value, 'posval')
+                assert r[0].args['pos'].value == 'posval'
 
-            @raises(ParseError)
             def omitted_positional_arg_raises_ParseError(self):
-                self._basic().parse_argv(['mytask'])
+                try:
+                    self._basic().parse_argv(['mytask'])
+                except ParseError as e:
+                    assert str(e) == "'mytask' did not receive required positional arguments: 'pos'" # noqa
+                else:
+                    assert False, "Did not raise ParseError!"
+
+            def omitted_positional_args_raises_ParseError(self):
+                try:
+                    arg = Argument('pos', positional=True)
+                    arg2 = Argument('morepos', positional=True)
+                    mytask = Context(name='mytask', args=[arg, arg2])
+                    Parser(contexts=[mytask]).parse_argv(['mytask'])
+                except ParseError as e:
+                    assert str(e) == "'mytask' did not receive required positional arguments: 'pos', 'morepos'" # noqa
+                else:
+                    assert False, "Did not raise ParseError!"
 
             def positional_args_eat_otherwise_valid_context_names(self):
                 mytask = Context('mytask', args=[
@@ -207,9 +222,9 @@ class Parser_(Spec):
                 Context('lolwut')
                 result = Parser([mytask]).parse_argv(['mytask', 'lolwut'])
                 r = result[0]
-                eq_(r.args['pos'].value, 'lolwut')
-                eq_(r.args['nonpos'].value, 'default')
-                eq_(len(result), 1) # Not 2
+                assert r.args['pos'].value == 'lolwut'
+                assert r.args['nonpos'].value == 'default'
+                assert len(result) == 1 # Not 2
 
             def positional_args_can_still_be_given_as_flags(self):
                 # AKA "positional args can come anywhere in the context"
@@ -217,22 +232,22 @@ class Parser_(Spec):
                 pos2 = Argument('pos2', positional=True)
                 nonpos = Argument('nonpos', positional=False, default='lol')
                 mytask = Context('mytask', args=[pos1, pos2, nonpos])
-                eq_(mytask.positional_args, [pos1, pos2])
+                assert mytask.positional_args == [pos1, pos2]
                 r = Parser([mytask]).parse_argv([
                     'mytask',
                     '--nonpos', 'wut',
                     '--pos2', 'pos2val',
                     'pos1val',
                 ])[0]
-                eq_(r.args['pos1'].value, 'pos1val')
-                eq_(r.args['pos2'].value, 'pos2val')
-                eq_(r.args['nonpos'].value, 'wut')
+                assert r.args['pos1'].value == 'pos1val'
+                assert r.args['pos2'].value == 'pos2val'
+                assert r.args['nonpos'].value == 'wut'
 
         class equals_signs:
             def _compare(self, argname, invoke, value):
                 c = Context('mytask', args=(Argument(argname, kind=str),))
                 r = Parser((c,)).parse_argv(['mytask', invoke])
-                eq_(r[0].args[argname].value, value)
+                assert r[0].args[argname].value == value
 
             def handles_equals_style_long_flags(self):
                 self._compare('foo', '--foo=bar', 'bar')
@@ -249,8 +264,8 @@ class Parser_(Spec):
             ))
             r = Parser([c]).parse_argv(['mytask', '--foo', '--bar'])
             a = r[0].args
-            eq_(a.foo.value, True)
-            eq_(a.bar.value, True)
+            assert a.foo.value is True
+            assert a.bar.value is True
 
     class optional_arg_values:
         def setup(self):
@@ -266,7 +281,8 @@ class Parser_(Spec):
                     ),
                 )
             self.context = Context('mytask', args=arguments)
-            return Parser([self.context])
+            self.parser = Parser([self.context])
+            return self.parser
 
         def _parse(self, argstr, parser=None):
             parser = parser or self.parser
@@ -274,7 +290,7 @@ class Parser_(Spec):
 
         def _expect(self, argstr, expected, parser=None):
             result = self._parse(argstr, parser)
-            eq_(result[0].args.foo.value, expected)
+            assert result[0].args.foo.value == expected
 
         def no_value_becomes_True_not_default_value(self):
             self._expect('--foo', True)
@@ -312,25 +328,45 @@ class Parser_(Spec):
                 ))
                 self._test_for_ambiguity("--foo uhoh", p)
 
-            def flaglike_value(self):
-                self._test_for_ambiguity("--foo --bar")
-
-            def no_ambiguity_with_flaglike_value_if_option_val_was_given(self):
+            def no_ambiguity_if_option_val_already_given(self):
                 p = self._parser((
                     Argument('foo', optional=True),
                     Argument('bar', kind=bool)
                 ))
                 # This should NOT raise a ParseError.
                 result = self._parse("--foo hello --bar", p)
-                eq_(result[0].args['foo'].value, 'hello')
-                eq_(result[0].args['bar'].value, True)
+                assert result[0].args['foo'].value == 'hello'
+                assert result[0].args['bar'].value is True
 
-            def actual_other_flag(self):
+            def valid_argument_is_NOT_ambiguous(self):
+                # The one exception that proves the rule?
                 self._parser((
                     Argument('foo', optional=True),
-                    Argument('bar')
+                    Argument('bar'),
                 ))
-                self._test_for_ambiguity("--foo --bar")
+                for form in ("--bar barval", "--bar=barval"):
+                    result = self._parse("--foo {}".format(form))
+                    assert len(result) == 1
+                    args = result[0].args
+                    assert args['foo'].value is True
+                    assert args['bar'].value == "barval"
+
+            def valid_flaglike_argument_is_NOT_ambiguous(self):
+                # The OTHER exception that proves the rule?
+                self._parser((
+                    Argument('foo', optional=True),
+                    Argument('bar', kind=bool),
+                ))
+                result = self._parse("--foo --bar")
+                assert len(result) == 1
+                args = result[0].args
+                assert args['foo'].value is True
+                assert args['bar'].value is True
+
+            def invalid_flaglike_value_is_stored_as_value(self):
+                self._parser((Argument('foo', optional=True),))
+                result = self._parse("--foo --bar")
+                assert result[0].args['foo'].value == "--bar"
 
             def task_name(self):
                 # mytask --foo myothertask
@@ -339,23 +375,110 @@ class Parser_(Spec):
                 p = Parser([c1, c2])
                 self._test_for_ambiguity("--foo othertask", p)
 
+    class list_type_arguments:
+        "list-type (iterable) arguments"
+        def _parse(self, *args):
+            c = Context('mytask', args=(Argument('mylist', kind=list),))
+            argv = ['mytask'] + list(args)
+            return Parser([c]).parse_argv(argv)[0].args.mylist.value
+
+        def can_be_given_no_times_resulting_in_default_empty_list(self):
+            assert self._parse() == []
+
+        def given_once_becomes_single_item_list(self):
+            assert self._parse('--mylist', 'foo') == ['foo']
+
+        def given_N_times_becomes_list_of_len_N(self):
+            expected = ['foo', 'bar', 'biz']
+            assert self._parse(
+                '--mylist', 'foo', '--mylist', 'bar', '--mylist', 'biz'
+            ) == expected
+
+        def iterables_work_correctly_outside_a_vacuum(self):
+            # Undetected bug where I was primarily focused on the -vvv use
+            # case...'normal' incrementables never left 'waiting for value'
+            # state in the parser! so _subsequent_ task names & such never got
+            # parsed right, always got appended to the list.
+            c = Context('mytask', args=[Argument('mylist', kind=list)])
+            c2 = Context('othertask')
+            argv = [
+                'mytask', '--mylist', 'val', '--mylist', 'val2', 'othertask'
+            ]
+            result = Parser([c, c2]).parse_argv(argv)
+            # When bug present, result only has one context (for 'mytask') and
+            # its 'mylist' consists of ['val', 'val2', 'othertask']. (the
+            # middle '--mylist' was handled semi-correctly.)
+            mylist = result[0].args.mylist.value
+            assert mylist == ['val', 'val2']
+            contexts = len(result)
+            err = "Got {} parse context results instead of 2!".format(contexts)
+            assert contexts == 2, err
+            assert result[1].name == 'othertask'
+
     class task_repetition:
         def is_happy_to_handle_same_task_multiple_times(self):
             task1 = Context('mytask')
             result = Parser((task1,)).parse_argv(['mytask', 'mytask'])
-            eq_(len(result), 2)
-            [eq_(x.name, 'mytask') for x in result]
+            assert len(result) == 2
+            for x in result:
+                assert x.name == 'mytask'
 
         def task_args_work_correctly(self):
             task1 = Context('mytask', args=(Argument('meh'),))
             result = Parser((task1,)).parse_argv(
                 ['mytask', '--meh', 'mehval1', 'mytask', '--meh', 'mehval2']
             )
-            eq_(result[0].args.meh.value, 'mehval1')
-            eq_(result[1].args.meh.value, 'mehval2')
+            assert result[0].args.meh.value == 'mehval1'
+            assert result[1].args.meh.value == 'mehval2'
+
+    class per_task_core_flags:
+        class help_:
+            def task_has_no_help_shows_per_task_help(self):
+                task1 = Context('mytask')
+                init = Context(args=[Argument('help', optional=True)])
+                parser = Parser(initial=init, contexts=[task1])
+                result = parser.parse_argv(['mytask', '--help'])
+                assert len(result) == 2
+                assert result[0].args.help.value == 'mytask'
+                assert 'help' not in result[1].args
+
+            # TODO: ideally we want an explosion, but for now, overriding
+            # happens naturally and is not the worst thing ever
+            def per_task_flag_wins_over_core_flag(self):
+                task1 = Context('mytask', args=[Argument('help')])
+                init = Context(args=[Argument('help', optional=True)])
+                parser = Parser(initial=init, contexts=[task1])
+                result = parser.parse_argv(['mytask', '--help', 'foo'])
+                assert result[1].args.help.value == 'foo'
+
+            def task_has_no_h_shortflag_shows_per_task_help(self):
+                task1 = Context('mytask')
+                arg = Argument(names=('help', 'h'), optional=True)
+                init = Context(args=[arg])
+                parser = Parser(initial=init, contexts=[task1])
+                result = parser.parse_argv(['mytask', '-h'])
+                assert len(result) == 2
+                assert result[0].args.help.value == 'mytask'
+                assert 'help' not in result[1].args
+
+            def task_has_h_shortflag_throws_error(self):
+                # def mytask(c, height):
+                # inv mytask -h
+                skip()
+
+        class other_core_flags_do_not_work_in_task_contexts:
+            # NOTE: only doing a subset here for sanity tests
+            def list_(self):
+                skip()
+
+            def no_dedupe(self):
+                skip()
+
+        # TODO: can define what core flags work as per-task flags, somehow?
+        # I.e. how to implement --roles/--hosts in fab 2?
 
 
-class ParseResult_(Spec):
+class ParseResult_:
     "ParseResult"
     def setup(self):
         self.context = Context('mytask',
@@ -364,8 +487,8 @@ class ParseResult_(Spec):
         self.result = Parser((self.context,)).parse_argv(argv)
 
     def acts_as_a_list_of_parsed_contexts(self):
-        eq_(len(self.result), 1)
-        eq_(self.result[0].name, 'mytask')
+        assert len(self.result) == 1
+        assert self.result[0].name == 'mytask'
 
     def exhibits_remainder_attribute(self):
-        eq_(self.result.remainder, 'my remainder')
+        assert self.result.remainder == 'my remainder'
